=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Designed November 2009 by Fredo6

# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:   MarkVertices6.rb
# Original Date	:   04 Nov 2009
# Description	:   Mark all vertices of the selection with a guide point
#                   This is seful to visualize the vertices
# Menu Item		:   Plugins > Triangulate DXF component or contextual menu on selection
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

require 'sketchup.rb'

module MarkVertices6

#Plugin installation - menu creation (done once)
def self.install_plugin
	
	#String Initialization
	case Sketchup.get_locale
	when /FR/i
		@menutitle = "Marquage de Vertex"
		@tooltip = "Marquage des vertex avec des points de construction"
		@msg_no_vertices = "Aucun vertex dans la selection"
		@msg_processing = "CALCUL"
		@msg_done = "Effectu\"
	else
		@menutitle = "Mark Vertices"
		@tooltip = "Mark vertices with Construction points"
		@msg_no_vertices = "NO vertex in the selection"
		@msg_processing = "PROCESSING"
		@msg_done = "DONE"
	end
	
	#Main menu
	sumenu = UI.menu "Plugins"
	#sumenu.add_separator
	cmd = UI::Command.new(@menutitle) { execute }
	cmd.status_bar_text = @tooltip
	sumenu.add_item cmd
	
	#Contextual menu
	UI.add_context_menu_handler do |menu|
		ss = Sketchup.active_model.selection
		if ss.find { |e| contains_vertex?(e)}
			menu.add_item(@menutitle) { execute }
		end	
	end 
	
end

#Check if the component is a DXF imported component
def self.contains_vertex?(e)
	e.instance_of?(Sketchup::Edge) || e.instance_of?(Sketchup::Face)
end

#Initialize and execute the marking of vertices
def self.execute(selection=nil)
	@hsh_vertices = {}
	@hsh_entities = {}
	@model = Sketchup.active_model
	selection = @model.selection unless selection
	
	#Create the list of vertices
	process_vertices selection
		
	#No vertices
	if !@hsh_vertices.values.find { |hsh| hsh.length > 0 }
		UI.messagebox "#{@msg_no_vertices}"
		return
	end
	
	#processing the components
	Sketchup.set_status_text @msg_processing, SB_VCB_LABEL
	@model.start_operation @menutitle

	n = 0
	@hsh_vertices.each do |key, hsh|
		entities = @hsh_entities[key]
		hsh.each { |k, pt| entities.add_cpoint pt }
		n += hsh.length
	end
	
	@model.commit_operation
	Sketchup.set_status_text @msg_done + " #{n}", SB_VCB_LABEL
end
		
def self.process_vertices(lentities, comp=nil)
	hsh = get_hash(comp)
	return unless hsh
	lentities.each do |e|
		if e.instance_of?(Sketchup::Edge)
			create_vertices hsh, [e.start, e.end]
		elsif e.instance_of?(Sketchup::Face)
			create_vertices hsh, e.vertices.to_a
		elsif e.instance_of?(Sketchup::Group)
			process_vertices e.entities, e
		elsif e.instance_of?(Sketchup::ComponentInstance)
			process_vertices e.definition.entities, e
		end
	end	
end
	
#Get the hash table to store vertices	
def self.get_hash(comp)
	if comp.instance_of?(Sketchup::ComponentInstance)
		key = comp.definition.entityID
		hsh = @hsh_vertices[key]
		return nil if hsh
		entities = comp.definition.entities
	elsif comp.instance_of?(Sketchup::Group)
		key = comp.entityID
		entities = comp.entities
	else	
		key = '__the__model'
		entities = @model.active_entities
	end	
	hsh = @hsh_vertices[key]
	return hsh if hsh
	
	hsh = @hsh_vertices[key] = {} unless hsh
	@hsh_entities[key] = entities
	hsh
end

#Create vertices	
def self.create_vertices(hsh, lvx)
	lvx.each do |vx|
		hsh[vx.entityID] = vx.position
	end
end
	
#----------------------------------------------------------------------------------------
# Menu and load for  Alternate Directory (called once)
#----------------------------------------------------------------------------------------

unless $mark_vertices____loaded
	self.install_plugin
	$mark_vertices____loaded = true
end

end	#module MarkVertices6